/*-----------------------------------------------------------------------------------
$Id: format_double.c,v 1.12 2012/03/06 08:54:23 wolf Exp wolf $
-----------------------------------------------------------------------------------*/

#include <stdlib.h>
#include <stdio.h>

#include "format_double.h"

#define MAXDIGITS  (18)
#define	quad	long long

#undef	USE_LLI

static quad quadExpTable[MAXDIGITS+1] = {
	1LL,
	10LL,
	100LL,
	1000LL,
	10000LL,
	100000LL,
	1000000LL,
	10000000LL,
	100000000LL,
	1000000000LL,
	10000000000LL,
	100000000000LL,
	1000000000000LL,
	10000000000000LL,
	100000000000000LL,
	1000000000000000LL,
	10000000000000000LL,
	100000000000000000LL,
	1000000000000000000LL
};

static t_format_result staticBuffer;

static int
min (int a, int b) {
	return a<b ? a : b;
}

const char*
formatDouble (t_format_result result, double value, unsigned length, unsigned fractDigits) {

	char* sign = "";
	quad expanded;
	quad integer, fract;
	unsigned integerDigits;
	char* p;
	int shift = 0;
	int origLength = length;

#ifndef USE_LLI
	char fractbuffer [20];
#endif

	/*-----------------------
	*	default buffer
	*----------------------*/

	if (!result) result = staticBuffer;

	/*-----------------------
	*	sanity: length
	*----------------------*/

	if (length ==0          ) length = sizeof(t_format_result)-1;
	if (length < 3          ) length = 3;
	if (length > sizeof(t_format_result)-1) length = sizeof(t_format_result)-1;

	/*-----------------------
	*	sanity: fract
	*----------------------*/

	if (fractDigits > MAXDIGITS) fractDigits = MAXDIGITS;
	if (fractDigits > length-3 ) fractDigits = length - 3;
	integerDigits = length - fractDigits - 1;

	/*-----------------------
	*	abs
	*----------------------*/

	if (value < 0) {
		value = -value;
		sign  = "-";
	}

	/*-----------------------
	*	scale
	*----------------------*/

	value = value * (double) quadExpTable[fractDigits] + 0.5;

	while (value >= 9223372036854775807.0 && shift < fractDigits) {
		shift++;
		value *= 0.1;
	}

	/*-----------------------
	*	general overflow?
	*----------------------*/

	if (value >= 9223372036854775807.0) {
		sprintf (result, "%*s", length, "***");
		return result;
	}

	/*-----------------------
	*	int+fract
	*----------------------*/

	expanded = (quad) value;
	integer = expanded / quadExpTable[fractDigits-shift];
	fract   = expanded % quadExpTable[fractDigits-shift] * quadExpTable[shift];

	/*-----------------------
	*	fixed overflow
	*----------------------*/

	if (integer >= quadExpTable[min(*sign ? integerDigits-1 : integerDigits, MAXDIGITS)]) {
		sprintf (result, "%*s", length, "***");
		return result;
	}

	/*-----------------------
	*	format
	*----------------------*/

#ifdef USE_LLI
	if (!origLength) {
		sprintf (result, "%s%lli.%0*lli", sign, integer, fractDigits, fract);
		return result;
	}
	sprintf (result, "%*lli.%0*lli", integerDigits, integer, fractDigits, fract);
#else
	if (fractDigits >= 10) {
		sprintf (fractbuffer, "%0*li%09li", fractDigits-9,
			(long) (fract/1000000000),
			(long) (fract%1000000000));
	} else {
		sprintf (fractbuffer, "%0*li", fractDigits, (long) fract);
	}

	if (integer >= 1000000000ll) {
		if (!origLength) {
			sprintf (result, "%s%li%09li.%s", sign,
				(long) (integer/1000000000),
				(long) (integer%1000000000),
				fractbuffer);
			return result;
		}
		sprintf (result, "%*li%09li.%s", integerDigits-9,
			(long) (integer/1000000000),
			(long) (integer%1000000000),
			fractbuffer);
	} else {
		if (!origLength) {
			sprintf (result, "%s%li.%s", sign, (long) integer, fractbuffer);
			return result;
		}
		sprintf (result, "%*li.%s", integerDigits, (long) integer, fractbuffer);
	}
#endif

	if (*sign) {
		p = result;
		while (p[1]==' ') p++;
		p[0] = *sign;
	}

	return result;
}

/*-----------------------------------------------------------------------------------
$Id: format_double.c,v 1.12 2012/03/06 08:54:23 wolf Exp wolf $
-----------------------------------------------------------------------------------*/
