/*-----------------------------------------------------------------------------------
$Id: regression.c,v 1.3 2011/11/04 13:04:06 wolf Exp $
-----------------------------------------------------------------------------------*/

#include "regression.h"

/*-----------------------------------------------------------------------------------
*		Initialisieren des Kontrollblocks
*----------------------------------------------------------------------------------*/

void
regressionInit (t_regression *rcb, int size, t_regression_xy buffer[]) {
	rcb->size 	= size;
	rcb->values 	= buffer;
	regressionReset (rcb);
}

/*-----------------------------------------------------------------------------------
*		Rücksetzen des Kontrollblocks
*----------------------------------------------------------------------------------*/

void
regressionReset (t_regression *rcb) {
	int i;
	rcb->n		= 0;
	rcb->sx		= 0.0;
	rcb->sy		= 0.0;
	rcb->sxx	= 0.0;
	rcb->sxy	= 0.0;
	rcb->last_x	= 0.0;
	rcb->valid	= 0;
	rcb->s		= 0.0;
	rcb->t		= 0.0;
	rcb->index	= 0;
	for (i=0; i<rcb->size; i++) {
		rcb->values[i].x = 0;
		rcb->values[i].y = 0;
	}
}

/*-----------------------------------------------------------------------------------
*		Neues Datenpaar einarbeiten
*----------------------------------------------------------------------------------*/

void
regressionAdd (t_regression *rcb, double x, double y) {

	/* entferne alten Wert */

	if (rcb->n >= rcb->size) {
		double	old_x	= rcb->values[rcb->index].x;
		double	old_y	= rcb->values[rcb->index].y;
		rcb->values[rcb->index].x = 0;
		rcb->values[rcb->index].y = 0;
		rcb->n		-= 1;
		rcb->sx		-= old_x;
		rcb->sy		-= old_y;
		rcb->sxx	-= old_x*old_x;
		rcb->sxy	-= old_x*old_y;
		rcb->valid	= 0;
		rcb->s		= 0.0;
		rcb->t		= 0.0;
	}

	if (rcb->n < rcb->size) {
		rcb->n		+= 1;
		rcb->sx		+= x;
		rcb->sy		+= y;
		rcb->sxx	+= x*x;
		rcb->sxy	+= x*y;
		rcb->valid	= 0;
		rcb->s		= 0.0;
		rcb->t		= 0.0;
		rcb->last_x	= x;
		rcb->values[rcb->index].x = x;
		rcb->values[rcb->index].y = y;
		rcb->index = (rcb->index+1) % rcb->size;
	}
}

/*-----------------------------------------------------------------------------------
*		Regression berechnen
*----------------------------------------------------------------------------------*/

static void
regressionCompute (t_regression *rcb) {

	double det;

	if (rcb->valid) return;
	rcb->valid = -1;

	det = rcb->n*rcb->sxx - rcb->sx*rcb->sx;

	/* TODO: hier möglicherweise ein abs(det)<epsilon */
	if (det==0.0) return;

	rcb->s = (rcb->n * rcb->sxy - rcb->sx * rcb->sy ) / det;
	rcb->t = (rcb->sxx*rcb->sy  - rcb->sx * rcb->sxy) / det;
	rcb->valid = 1;
}

/*-----------------------------------------------------------------------------------
*		Über Regression berechneter y-Wert zum letzten x-Wert
*----------------------------------------------------------------------------------*/

double
regressionActual (t_regression *rcb) {
	regressionCompute (rcb);
	return rcb->last_x * rcb->s + rcb->t;
}

/*-----------------------------------------------------------------------------------
*		Über Regression berechneter y-Wert zum angegeben x-Wert
*----------------------------------------------------------------------------------*/

double
regressionForecast (t_regression *rcb, double x) {
	regressionCompute (rcb);
	return x * rcb->s + rcb->t;
}

/*-----------------------------------------------------------------------------------
*		Über Regression berechneter x-Wert zum angegeben y-Wert
*----------------------------------------------------------------------------------*/

double
regressionReverse (t_regression *rcb, double y) {
	regressionCompute (rcb);
	return rcb->s != 0.0 ? (y - rcb->t) / rcb->s : 1e9;
}

/*-----------------------------------------------------------------------------------
*		Steigung der Regressionsgrade, also Änderung von y bei Änderung von x
*----------------------------------------------------------------------------------*/

double
regressionChange (t_regression *rcb) {
	regressionCompute (rcb);
	return rcb->s;
}

/*-----------------------------------------------------------------------------------
*		Qualität der Regression (0..1)
*	(vereinfachte Implementierung, liefert 1, wenn Regression verfügbar, 0 sonst)
*	(vollständige Implementierung wird nachgeliefert)
*----------------------------------------------------------------------------------*/

double
regressionQuality (t_regression *rcb) {
	regressionCompute (rcb);
	return rcb->valid==1 ? 1.0 : 0.0;
}

/*-----------------------------------------------------------------------------------
$Id: regression.c,v 1.3 2011/11/04 13:04:06 wolf Exp $
-----------------------------------------------------------------------------------*/
