/*-----------------------------------------------------------------------------------
$Id: regression.h,v 1.4 2012/03/06 08:54:23 wolf Exp wolf $
-----------------------------------------------------------------------------------*/

/*
Anzahl der "Stützstellen".
Wenn nicht definiert, muss bei der Initialisierung des Kontrollblocks
die Anzahl und ein Zeiger auf ein double-Array dieser Größe übergeben werden.
*/

/*
Kontrollblock für die Berechnung
*/

typedef struct {
	double	x;
	double	y;
} t_regression_xy;

typedef struct {
	/* buffer */
	int	size;
	t_regression_xy* values;
	/* storage */
	int	index;
	/* sums */
	double	n;
	double	sx;
	double	sy;
	double	sxy;
	double	sxx;
	double	last_x;
	/* result */
	int	valid;
	double	s;
	double	t;
} t_regression;

/*-----------------------------------------------------------------------------------
*		Initialisieren des Kontrollblocks
*----------------------------------------------------------------------------------*/

extern void
regressionInit (t_regression *rcb, int size, t_regression_xy buffer[]);

/*-----------------------------------------------------------------------------------
*		Rücksetzen des Kontrollblocks
*----------------------------------------------------------------------------------*/

extern void
regressionReset (t_regression *rcb);

/*-----------------------------------------------------------------------------------
*		Neues Datenpaar einarbeiten
*----------------------------------------------------------------------------------*/

extern void
regressionAdd (t_regression *rcb, double x, double y);

/*-----------------------------------------------------------------------------------
*		Über Regression berechneter y-Wert zum letzten x-Wert
*----------------------------------------------------------------------------------*/

extern double
regressionActual (t_regression *rcb);

/*-----------------------------------------------------------------------------------
*		Über Regression berechneter y-Wert zum angegeben x-Wert
*----------------------------------------------------------------------------------*/

extern double
regressionForecast (t_regression *rcb, double x);

/*-----------------------------------------------------------------------------------
*		Über Regression berechneter x-Wert zum angegeben y-Wert
*----------------------------------------------------------------------------------*/

extern double
regressionReverse (t_regression *rcb, double y);

/*-----------------------------------------------------------------------------------
*		Steigung der Regressionsgrade, also Änderung von y bei Änderung von x
*----------------------------------------------------------------------------------*/

extern double
regressionChange (t_regression *rcb);

/*-----------------------------------------------------------------------------------
*		Qualität der Regression (0.0…1.0)
*----------------------------------------------------------------------------------*/

extern double
regressionQuality (t_regression *rcb);

/*-----------------------------------------------------------------------------------
$Id: regression.h,v 1.4 2012/03/06 08:54:23 wolf Exp wolf $
-----------------------------------------------------------------------------------*/
